package com.bokisoftware.axis;

/*
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.util.ArrayList;

import android.opengl.GLES20;
import android.opengl.Matrix;

/**
 * Generates and draws the X, Y and Z axis along with the axis tick marks.
 * 
 * @author Kaleb
 * @version 1.0
 * 
 */
public class Axis
{
	// x-axis handlers
	private int axesXPositionHandle;
	private int axesXColorHandle;
	private int axesXNormalHandle;

	// x-axis buffers
	private FloatBuffer axesXVertexBuffer;
	private FloatBuffer axesXNormalsBuffer;
	private FloatBuffer axesXColorBuffer;

	// y-axis handlers
	private int axesYPositionHandle;
	private int axesYColorHandle;
	private int axesYNormalHandle;

	// y-axis buffers
	private FloatBuffer axesYVertexBuffer;
	private FloatBuffer axesYNormalsBuffer;
	private FloatBuffer axesYColorBuffer;

	// z-axis handlers
	private int axesZPositionHandle;
	private int axesZColorHandle;
	private int axesZNormalHandle;

	// z-axis buffers
	private FloatBuffer axesZVertexBuffer;
	private FloatBuffer axesZNormalsBuffer;
	private FloatBuffer axesZColorBuffer;

	// x-ticks handlers
	private int ticksXPositionHandle;
	private int ticksXColorHandle;
	private int ticksXNormalHandle;

	// x-ticks buffers
	private FloatBuffer ticksXVertexBuffer;
	private FloatBuffer ticksXNormalsBuffer;
	private FloatBuffer ticksXColorBuffer;

	// y-ticks handlers
	private int ticksYPositionHandle;
	private int ticksYColorHandle;
	private int ticksYNormalHandle;

	// y-ticks buffers
	private FloatBuffer ticksYVertexBuffer;
	private FloatBuffer ticksYNormalsBuffer;
	private FloatBuffer ticksYColorBuffer;

	// z-ticks handlers
	private int ticksZPositionHandle;
	private int ticksZColorHandle;
	private int ticksZNormalHandle;

	// z-ticks buffers
	private FloatBuffer ticksZVertexBuffer;
	private FloatBuffer ticksZNormalsBuffer;
	private FloatBuffer ticksZColorBuffer;

	// the number of axis points
	private int axesXPoints;
	private int axesYPoints;
	private int axesZPoints;

	// the number of tick points
	private int ticksXPoints;
	private int ticksYPoints;
	private int ticksZPoints;

	// This will be used to pass in the transformation matrix.
	private int mvpMatrixHandle;

	// This will be used to pass in the modelview matrix.
	private int mvMatrixHandle;

	// This will be used to pass in the light position.
	private int lightPosHandle;

	// How many bytes per float.
	private final int bytesPerFloat = 4;

	// Size of the position data in elements.
	private final int positionDataSize = 3;

	// Size of the color data in elements.
	private final int colorDataSize = 4;

	// Size of the normal data in elements.
	private final int normalsDataSize = 3;

	// This is a handle to our per-vertex cube shading program.
	private int lineProgramHandle;

	// The bounds of the axis.
	private int bounds = 3;

	/**
	 * Create a new instance of Axes.
	 */
	public Axis()
	{
		// Create the axis
		generateXAxis();
		generateYAxis();
		generateZAxis();

		// Create the ticks
		generateXTicks();
		generateYTicks();
		generateZTicks();
	}

	/**
	 * Draw the axis.
	 * 
	 * @param mvpMatrix
	 *            the model-view-perspective
	 * @param viewMatrix
	 *            the view matrix
	 * @param projectionMatrix
	 *            the projection matrix
	 * @param modelMatrix
	 *            the model matrix
	 * @param lightPosInEyeSpace
	 *            the light position in eye space
	 */
	public void drawAxes(float[] mvpMatrix, float[] viewMatrix,
			float[] projectionMatrix, float[] modelMatrix,
			float[] lightPosInEyeSpace)
	{

		// Draw the axis
		drawXAxis(mvpMatrix, viewMatrix, projectionMatrix, modelMatrix,
				lightPosInEyeSpace);
		drawYAxis(mvpMatrix, viewMatrix, projectionMatrix, modelMatrix,
				lightPosInEyeSpace);
		drawZAxis(mvpMatrix, viewMatrix, projectionMatrix, modelMatrix,
				lightPosInEyeSpace);

		// Draw the ticks
		drawXAxisTicks(mvpMatrix, viewMatrix, projectionMatrix, modelMatrix,
				lightPosInEyeSpace);
		drawYAxisTicks(mvpMatrix, viewMatrix, projectionMatrix, modelMatrix,
				lightPosInEyeSpace);
		drawZAxisTicks(mvpMatrix, viewMatrix, projectionMatrix, modelMatrix,
				lightPosInEyeSpace);
	}

	/**
	 * Set the handlers for the shaders.
	 */
	public void setHandles()
	{
		// set x-axis handles.
		axesXColorHandle = (GLES20.glGetAttribLocation(lineProgramHandle,
				"a_Color"));
		mvpMatrixHandle = GLES20.glGetUniformLocation(lineProgramHandle,
				"u_MVPMatrix");
		axesXPositionHandle = GLES20.glGetAttribLocation(lineProgramHandle,
				"a_Position");

		// set y-axis handles.
		axesYColorHandle = (GLES20.glGetAttribLocation(lineProgramHandle,
				"a_Color"));
		mvpMatrixHandle = GLES20.glGetUniformLocation(lineProgramHandle,
				"u_MVPMatrix");
		axesYPositionHandle = GLES20.glGetAttribLocation(lineProgramHandle,
				"a_Position");

		// set z-axis handles.
		axesZColorHandle = (GLES20.glGetAttribLocation(lineProgramHandle,
				"a_Color"));
		mvpMatrixHandle = GLES20.glGetUniformLocation(lineProgramHandle,
				"u_MVPMatrix");
		axesZPositionHandle = GLES20.glGetAttribLocation(lineProgramHandle,
				"a_Position");

		// set x-ticks handles.
		ticksXColorHandle = (GLES20.glGetAttribLocation(lineProgramHandle,
				"a_Color"));
		mvpMatrixHandle = GLES20.glGetUniformLocation(lineProgramHandle,
				"u_MVPMatrix");
		ticksXPositionHandle = GLES20.glGetAttribLocation(lineProgramHandle,
				"a_Position");

		// set y-ticks handles.
		ticksYColorHandle = (GLES20.glGetAttribLocation(lineProgramHandle,
				"a_Color"));
		mvpMatrixHandle = GLES20.glGetUniformLocation(lineProgramHandle,
				"u_MVPMatrix");
		ticksYPositionHandle = GLES20.glGetAttribLocation(lineProgramHandle,
				"a_Position");

		// set y-ticks handles.
		ticksZColorHandle = (GLES20.glGetAttribLocation(lineProgramHandle,
				"a_Color"));
		mvpMatrixHandle = GLES20.glGetUniformLocation(lineProgramHandle,
				"u_MVPMatrix");
		ticksZPositionHandle = GLES20.glGetAttribLocation(lineProgramHandle,
				"a_Position");
	}

	/**
	 * Set the program handle for the shaders.
	 * 
	 * @param lineProgramHandle
	 *            the shader program handle.
	 */
	public void setLineProgramHandle(int lineProgramHandle)
	{
		this.lineProgramHandle = lineProgramHandle;
	}

	/**
	 * Generate the ticks on the x-axis.
	 */
	private void generateXTicks()
	{
		// The length of the tick in the positive and negative axis.
		float tickLength = 0.1f;
		// The distance between ticks.
		float tickPadding = 0.1f;

		// The number of ticks.
		ticksXPoints = (int) ((4 * Math.abs(bounds)) / tickPadding) + 2;

		ArrayList<Float> ticksXCoordsList = new ArrayList<Float>();
		ArrayList<Float> ticksXColorsList = new ArrayList<Float>();

		// Start on the negative axis and move draw ticks towards the positive
		// axis.
		float location = -bounds;
		// Create the ticks.
		for (int i = 0; i < ticksXPoints; i++)
		{
			// Tick in positive axis.
			ticksXCoordsList.add(tickLength);
			ticksXCoordsList.add(0.0f);
			ticksXCoordsList.add(location);

			// Tick in negative axis.
			ticksXCoordsList.add(-tickLength);
			ticksXCoordsList.add(0.0f);
			ticksXCoordsList.add(location);

			// Positive axis color.
			// RGBA
			ticksXColorsList.add(1.0f);
			ticksXColorsList.add(1.0f);
			ticksXColorsList.add(1.0f);
			ticksXColorsList.add(1.0f);

			// Negative axis color.
			// RGBA
			ticksXColorsList.add(1.0f);
			ticksXColorsList.add(1.0f);
			ticksXColorsList.add(1.0f);
			ticksXColorsList.add(1.0f);

			// Increment the location by the padding distance.
			location += tickPadding;
		}

		// Create arrays for the buffers.
		float[] ticksXCoordsArray = new float[ticksXCoordsList.size()];
		float[] ticksXColorsArray = new float[ticksXColorsList.size()];

		// Transfer the list contents to the array.
		for (int i = 0; i < ticksXCoordsArray.length; i++)
		{
			ticksXCoordsArray[i] = ticksXCoordsList.get(i);
		}

		// Transfer the list contents to the array.
		for (int i = 0; i < ticksXColorsArray.length; i++)
		{
			ticksXColorsArray[i] = ticksXColorsList.get(i);
		}

		// Create the x-axis vertex (points) buffer.
		ticksXVertexBuffer = ByteBuffer
				.allocateDirect(ticksXCoordsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		ticksXVertexBuffer.put(ticksXCoordsArray).position(0);

		// Create the x-axis normals buffer.
		ticksXNormalsBuffer = ByteBuffer
				.allocateDirect(ticksXCoordsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		ticksXNormalsBuffer.put(ticksXCoordsArray).position(0);

		// Create the x-axis color buffer.
		ticksXColorBuffer = ByteBuffer
				.allocateDirect(ticksXColorsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		ticksXColorBuffer.put(ticksXColorsArray).position(0);
	}

	/**
	 * Generate the ticks on the y-axis.
	 */
	private void generateYTicks()
	{
		// The length of the tick in the positive and negative axis.
		float tickLength = 0.1f;
		// The distance between ticks.
		float tickPadding = 0.1f;

		// The number of ticks.
		ticksYPoints = (int) ((4 * Math.abs(bounds)) / tickPadding) + 2;

		ArrayList<Float> ticksYCoordsList = new ArrayList<Float>();
		ArrayList<Float> ticksYColorsList = new ArrayList<Float>();

		// Start on the negative axis and move draw ticks towards the positive
		// axis.
		float location = -bounds;
		// Create the ticks.
		for (int i = 0; i < ticksXPoints; i++)
		{
			// Tick in positive axis.
			ticksYCoordsList.add(tickLength);
			ticksYCoordsList.add(location);
			ticksYCoordsList.add(0.0f);

			// Tick in negative axis.
			ticksYCoordsList.add(-tickLength);
			ticksYCoordsList.add(location);
			ticksYCoordsList.add(0.0f);

			// Positive axis color.
			// RGBA
			ticksYColorsList.add(1.0f);
			ticksYColorsList.add(1.0f);
			ticksYColorsList.add(1.0f);
			ticksYColorsList.add(1.0f);

			// Negative axis color.
			// RGBA
			ticksYColorsList.add(1.0f);
			ticksYColorsList.add(1.0f);
			ticksYColorsList.add(1.0f);
			ticksYColorsList.add(1.0f);

			// Increment the location by the padding distance.
			location += tickPadding;
		}

		// Create arrays for the buffers.
		float[] ticksYCoordsArray = new float[ticksYCoordsList.size()];
		float[] ticksYColorsArray = new float[ticksYColorsList.size()];

		// Transfer the list contents to the array.
		for (int i = 0; i < ticksYCoordsArray.length; i++)
		{
			ticksYCoordsArray[i] = ticksYCoordsList.get(i);
		}

		// Transfer the list contents to the array.
		for (int i = 0; i < ticksYColorsArray.length; i++)
		{
			ticksYColorsArray[i] = ticksYColorsList.get(i);
		}

		// Create the y-axis vertex (points) buffer.
		ticksYVertexBuffer = ByteBuffer
				.allocateDirect(ticksYCoordsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		ticksYVertexBuffer.put(ticksYCoordsArray).position(0);

		// Create the y-axis normals buffer.
		ticksYNormalsBuffer = ByteBuffer
				.allocateDirect(ticksYCoordsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		ticksYNormalsBuffer.put(ticksYCoordsArray).position(0);

		// Create the y-axis colors buffer.
		ticksYColorBuffer = ByteBuffer
				.allocateDirect(ticksYColorsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		ticksYColorBuffer.put(ticksYColorsArray).position(0);
	}

	/**
	 * Generate the ticks on the z-axis.
	 */
	private void generateZTicks()
	{
		// The length of the tick in the positive and negative axis.
		float tickLength = 0.1f;
		// The distance between ticks.
		float tickPadding = 0.1f;

		// The number of ticks.
		ticksZPoints = (int) ((4 * Math.abs(bounds)) / tickPadding) + 2;

		ArrayList<Float> ticksZCoordsList = new ArrayList<Float>();
		ArrayList<Float> ticksZColorsList = new ArrayList<Float>();

		// Start on the negative axis and move draw ticks towards the positive
		// axis.
		float location = -bounds;
		// Create the ticks.
		for (int i = 0; i < ticksZPoints; i++)
		{
			// Tick in positive axis.
			ticksZCoordsList.add(location);
			ticksZCoordsList.add(tickLength);
			ticksZCoordsList.add(0.0f);

			// Tick in negative axis.
			ticksZCoordsList.add(location);
			ticksZCoordsList.add(-tickLength);
			ticksZCoordsList.add(0.0f);

			// Positive axis color.
			// RGBA
			ticksZColorsList.add(1.0f);
			ticksZColorsList.add(1.0f);
			ticksZColorsList.add(1.0f);
			ticksZColorsList.add(1.0f);

			// Negative axis color.
			// RGBA
			ticksZColorsList.add(1.0f);
			ticksZColorsList.add(1.0f);
			ticksZColorsList.add(1.0f);
			ticksZColorsList.add(1.0f);

			// Increment the location by the padding distance.
			location += tickPadding;
		}

		// Create arrays for the buffers.
		float[] ticksZCoordsArray = new float[ticksZCoordsList.size()];
		float[] ticksZColorsArray = new float[ticksZColorsList.size()];

		// Transfer the list contents to the array.
		for (int i = 0; i < ticksZCoordsArray.length; i++)
		{
			ticksZCoordsArray[i] = ticksZCoordsList.get(i);
		}

		// Transfer the list contents to the array.
		for (int i = 0; i < ticksZColorsArray.length; i++)
		{
			ticksZColorsArray[i] = ticksZColorsList.get(i);
		}

		// Create the z-axis vertex (points) buffer.
		ticksZVertexBuffer = ByteBuffer
				.allocateDirect(ticksZCoordsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		ticksZVertexBuffer.put(ticksZCoordsArray).position(0);

		// Create the z-axis normals buffer.
		ticksZNormalsBuffer = ByteBuffer
				.allocateDirect(ticksZCoordsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		ticksZNormalsBuffer.put(ticksZCoordsArray).position(0);

		// Create the z-axis color buffer.
		ticksZColorBuffer = ByteBuffer
				.allocateDirect(ticksZColorsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		ticksZColorBuffer.put(ticksZColorsArray).position(0);
	}

	/**
	 * Generate the points for the x-axis.
	 */
	private void generateXAxis()
	{
		// Create the two points for the line.
		float[] axesXCoordsArray =
		{ bounds, 0, 0, -bounds, 0, 0 };

		// The number of points.
		axesXPoints = axesXCoordsArray.length / 3;

		// Create the x-axis vertex buffer.
		axesXVertexBuffer = ByteBuffer
				.allocateDirect(axesXCoordsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		axesXVertexBuffer.put(axesXCoordsArray).position(0);

		// Create the two points for the normals.
		float[] axesXNormalsArray =
		{ bounds, 0, 0, -bounds, 0, 0 };

		// Create the x-axis normals buffer.
		axesXNormalsBuffer = ByteBuffer
				.allocateDirect(axesXNormalsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		axesXNormalsBuffer.put(axesXNormalsArray).position(0);

		// Create two points for the color.
		float[] axesXColorsArray =
		{ 1, 1, 1, 1, 1, 1, 1, 1 };

		// Create the x-axis colors buffer.
		axesXColorBuffer = ByteBuffer
				.allocateDirect(axesXColorsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		axesXColorBuffer.put(axesXColorsArray).position(0);

	}

	/**
	 * Generate the points for the y-axis.
	 */
	private void generateYAxis()
	{
		// Create two points for the line/
		float[] axesYCoordsArray =
		{ 0, bounds, 0, 0, -bounds, 0 };

		// The number of points.
		axesYPoints = axesYCoordsArray.length / 3;

		// Create the y-axis vertex buffer.
		axesYVertexBuffer = ByteBuffer
				.allocateDirect(axesYCoordsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		axesYVertexBuffer.put(axesYCoordsArray).position(0);

		// Create two points for the normals.
		float[] axesYNormalsArray =
		{ 0, bounds, 0, 0, -bounds, 0 };

		// Create the y-axis normals point buffer.
		axesYNormalsBuffer = ByteBuffer
				.allocateDirect(axesYNormalsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		axesYNormalsBuffer.put(axesYNormalsArray).position(0);

		// Create two points for the color.
		float[] axesYColorsArray =
		{ 1, 1, 1, 1, 1, 1, 1, 1 };

		// Create the y-axis color buffer.
		axesYColorBuffer = ByteBuffer
				.allocateDirect(axesYColorsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		axesYColorBuffer.put(axesYColorsArray).position(0);
	}

	/**
	 * Generate the points for the z-axis.
	 */
	private void generateZAxis()
	{
		// Create two points for the z-axis line.
		float[] axesZCoordsArray =
		{ 0, 0, bounds, 0, 0, -bounds };

		// The number of points.
		axesZPoints = axesZCoordsArray.length / 3;

		// Create the z-axis vertex buffer.
		axesZVertexBuffer = ByteBuffer
				.allocateDirect(axesZCoordsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		axesZVertexBuffer.put(axesZCoordsArray).position(0);

		// Create two points for the z-axis normals.
		float[] axesZNormalsArray =
		{ 0, 0, bounds, 0, 0, -bounds };

		// Create the z-axis normals buffer.
		axesZNormalsBuffer = ByteBuffer
				.allocateDirect(axesZNormalsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		axesZNormalsBuffer.put(axesZNormalsArray).position(0);

		// Create two points for the z-axis color.
		float[] axesZColorsArray =
		{ 1, 1, 1, 1, 1, 1, 1, 1 };

		// Create the z-axis colors buffer.
		axesZColorBuffer = ByteBuffer
				.allocateDirect(axesZColorsArray.length * bytesPerFloat)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();
		axesZColorBuffer.put(axesZColorsArray).position(0);
	}

	/**
	 * Draw the x-axis ticks.
	 * 
	 * @param mvpMatrix
	 *            the model-view-perspective matrix
	 * @param viewMatrix
	 *            the view matrix
	 * @param projectionMatrix
	 *            the projection matrix
	 * @param modelMatrix
	 *            the model matrix
	 * @param lightPosInEyeSpace
	 *            the light position in eye space matrix
	 */
	private void drawXAxisTicks(float[] mvpMatrix, float[] viewMatrix,
			float[] projectionMatrix, float[] modelMatrix,
			float[] lightPosInEyeSpace)
	{
		// Draw the ticks on the x-axis.
		GLES20.glUseProgram(lineProgramHandle);

		// Pass in the position information
		ticksXVertexBuffer.position(0);
		GLES20.glVertexAttribPointer(ticksXPositionHandle, positionDataSize,
				GLES20.GL_FLOAT, false, 0, ticksXVertexBuffer);

		GLES20.glEnableVertexAttribArray(ticksXPositionHandle);

		// Pass in the color information
		ticksXColorBuffer.position(0);
		GLES20.glVertexAttribPointer(ticksXColorHandle, colorDataSize,
				GLES20.GL_FLOAT, false, 0, ticksXColorBuffer);

		GLES20.glEnableVertexAttribArray(ticksXColorHandle);

		// Pass in the normal information
		ticksXNormalsBuffer.position(0);
		GLES20.glVertexAttribPointer(ticksXNormalHandle, normalsDataSize,
				GLES20.GL_FLOAT, false, 0, ticksXNormalsBuffer);

		GLES20.glEnableVertexAttribArray(ticksXNormalHandle);

		// This multiplies the view matrix by the model matrix, and stores the
		// result in the MVP matrix
		// (which currently contains model * view).
		Matrix.multiplyMM(mvpMatrix, 0, viewMatrix, 0, modelMatrix, 0);

		// Pass in the modelview matrix.
		GLES20.glUniformMatrix4fv(mvMatrixHandle, 1, false, mvpMatrix, 0);

		// This multiplies the modelview matrix by the projection matrix, and
		// stores the result in the MVP matrix
		// (which now contains model * view * projection).
		Matrix.multiplyMM(mvpMatrix, 0, projectionMatrix, 0, mvpMatrix, 0);

		// Pass in the combined matrix.
		GLES20.glUniformMatrix4fv(mvpMatrixHandle, 1, false, mvpMatrix, 0);

		// Pass in the light position in eye space.
		GLES20.glUniform3f(lightPosHandle, lightPosInEyeSpace[0],
				lightPosInEyeSpace[1], lightPosInEyeSpace[2]);

		GLES20.glLineWidth(1);

		// Draw the x-axis ticks.
		GLES20.glDrawArrays(GLES20.GL_LINES, 0, ticksXPoints);
	}

	/**
	 * Draw the y-axis ticks.
	 * 
	 * @param mvpMatrix
	 *            the model-view-perspective matrix
	 * @param viewMatrix
	 *            the view matrix
	 * @param projectionMatrix
	 *            the projection matrix
	 * @param modelMatrix
	 *            the model matrix
	 * @param lightPosInEyeSpace
	 *            the light position in eye space matrix
	 */
	private void drawYAxisTicks(float[] mvpMatrix, float[] viewMatrix,
			float[] projectionMatrix, float[] modelMatrix,
			float[] lightPosInEyeSpace)
	{
		// Draw the y-axis ticks.
		GLES20.glUseProgram(lineProgramHandle);

		// Pass in the position information
		ticksYVertexBuffer.position(0);
		GLES20.glVertexAttribPointer(ticksYPositionHandle, positionDataSize,
				GLES20.GL_FLOAT, false, 0, ticksYVertexBuffer);

		GLES20.glEnableVertexAttribArray(ticksYPositionHandle);

		// Pass in the color information
		ticksYColorBuffer.position(0);
		GLES20.glVertexAttribPointer(ticksYColorHandle, colorDataSize,
				GLES20.GL_FLOAT, false, 0, ticksYColorBuffer);

		GLES20.glEnableVertexAttribArray(ticksYColorHandle);

		// Pass in the normal information
		ticksYNormalsBuffer.position(0);
		GLES20.glVertexAttribPointer(ticksYNormalHandle, normalsDataSize,
				GLES20.GL_FLOAT, false, 0, ticksYNormalsBuffer);

		GLES20.glEnableVertexAttribArray(ticksYNormalHandle);

		// This multiplies the view matrix by the model matrix, and stores the
		// result in the MVP matrix
		// (which currently contains model * view).
		Matrix.multiplyMM(mvpMatrix, 0, viewMatrix, 0, modelMatrix, 0);

		// Pass in the modelview matrix.
		GLES20.glUniformMatrix4fv(mvMatrixHandle, 1, false, mvpMatrix, 0);

		// This multiplies the modelview matrix by the projection matrix, and
		// stores the result in the MVP matrix
		// (which now contains model * view * projection).
		Matrix.multiplyMM(mvpMatrix, 0, projectionMatrix, 0, mvpMatrix, 0);

		// Pass in the combined matrix.
		GLES20.glUniformMatrix4fv(mvpMatrixHandle, 1, false, mvpMatrix, 0);

		// Pass in the light position in eye space.
		GLES20.glUniform3f(lightPosHandle, lightPosInEyeSpace[0],
				lightPosInEyeSpace[1], lightPosInEyeSpace[2]);

		GLES20.glLineWidth(1);

		// Draw the y-axis ticks.
		GLES20.glDrawArrays(GLES20.GL_LINES, 0, ticksYPoints);
	}

	/**
	 * Draw the z-axis ticks.
	 * 
	 * @param mvpMatrix
	 *            the model-view-perspective matrix
	 * @param viewMatrix
	 *            the view matrix
	 * @param projectionMatrix
	 *            the projection matrix
	 * @param modelMatrix
	 *            the model matrix
	 * @param lightPosInEyeSpace
	 *            the light position in eye space matrix
	 */
	private void drawZAxisTicks(float[] mvpMatrix, float[] viewMatrix,
			float[] projectionMatrix, float[] modelMatrix,
			float[] lightPosInEyeSpace)
	{
		// Draw the z-axis ticks.
		GLES20.glUseProgram(lineProgramHandle);

		// Pass in the position information
		ticksZVertexBuffer.position(0);
		GLES20.glVertexAttribPointer(ticksZPositionHandle, positionDataSize,
				GLES20.GL_FLOAT, false, 0, ticksZVertexBuffer);

		GLES20.glEnableVertexAttribArray(ticksZPositionHandle);

		// Pass in the color information
		ticksZColorBuffer.position(0);
		GLES20.glVertexAttribPointer(ticksZColorHandle, colorDataSize,
				GLES20.GL_FLOAT, false, 0, ticksZColorBuffer);

		GLES20.glEnableVertexAttribArray(ticksZColorHandle);

		// Pass in the normal information
		ticksZNormalsBuffer.position(0);
		GLES20.glVertexAttribPointer(ticksZNormalHandle, normalsDataSize,
				GLES20.GL_FLOAT, false, 0, ticksZNormalsBuffer);

		GLES20.glEnableVertexAttribArray(ticksZNormalHandle);

		// This multiplies the view matrix by the model matrix, and stores the
		// result in the MVP matrix
		// (which currently contains model * view).
		Matrix.multiplyMM(mvpMatrix, 0, viewMatrix, 0, modelMatrix, 0);

		// Pass in the modelview matrix.
		GLES20.glUniformMatrix4fv(mvMatrixHandle, 1, false, mvpMatrix, 0);

		// This multiplies the modelview matrix by the projection matrix, and
		// stores the result in the MVP matrix
		// (which now contains model * view * projection).
		Matrix.multiplyMM(mvpMatrix, 0, projectionMatrix, 0, mvpMatrix, 0);

		// Pass in the combined matrix.
		GLES20.glUniformMatrix4fv(mvpMatrixHandle, 1, false, mvpMatrix, 0);

		// Pass in the light position in eye space.
		GLES20.glUniform3f(lightPosHandle, lightPosInEyeSpace[0],
				lightPosInEyeSpace[1], lightPosInEyeSpace[2]);

		GLES20.glLineWidth(1);

		// Draw the z-axis ticks.
		GLES20.glDrawArrays(GLES20.GL_LINES, 0, ticksZPoints);
	}

	/**
	 * Draw the x-axis.
	 * 
	 * @param mvpMatrix
	 *            the model-view-perspective matrix
	 * @param viewMatrix
	 *            the view matrix
	 * @param projectionMatrix
	 *            the projection matrix
	 * @param modelMatrix
	 *            the model matrix
	 * @param lightPosInEyeSpace
	 *            the light position in eye space matrix
	 */
	private void drawXAxis(float[] mvpMatrix, float[] viewMatrix,
			float[] projectionMatrix, float[] modelMatrix,
			float[] lightPosInEyeSpace)
	{
		// Draw the x-axis.
		GLES20.glUseProgram(lineProgramHandle);

		// Pass in the position information
		axesXVertexBuffer.position(0);
		GLES20.glVertexAttribPointer(axesXPositionHandle, positionDataSize,
				GLES20.GL_FLOAT, false, 0, axesXVertexBuffer);

		GLES20.glEnableVertexAttribArray(axesXPositionHandle);

		// Pass in the color information
		axesXColorBuffer.position(0);
		GLES20.glVertexAttribPointer(axesXColorHandle, colorDataSize,
				GLES20.GL_FLOAT, false, 0, axesXColorBuffer);

		GLES20.glEnableVertexAttribArray(axesXColorHandle);

		// Pass in the normal information
		axesXNormalsBuffer.position(0);
		GLES20.glVertexAttribPointer(axesXNormalHandle, normalsDataSize,
				GLES20.GL_FLOAT, false, 0, axesXNormalsBuffer);

		GLES20.glEnableVertexAttribArray(axesXNormalHandle);

		// This multiplies the view matrix by the model matrix, and stores the
		// result in the MVP matrix
		// (which currently contains model * view).
		Matrix.multiplyMM(mvpMatrix, 0, viewMatrix, 0, modelMatrix, 0);

		// Pass in the modelview matrix.
		GLES20.glUniformMatrix4fv(mvMatrixHandle, 1, false, mvpMatrix, 0);

		// This multiplies the modelview matrix by the projection matrix, and
		// stores the result in the MVP matrix
		// (which now contains model * view * projection).
		Matrix.multiplyMM(mvpMatrix, 0, projectionMatrix, 0, mvpMatrix, 0);

		// Pass in the combined matrix.
		GLES20.glUniformMatrix4fv(mvpMatrixHandle, 1, false, mvpMatrix, 0);

		// Pass in the light position in eye space.
		GLES20.glUniform3f(lightPosHandle, lightPosInEyeSpace[0],
				lightPosInEyeSpace[1], lightPosInEyeSpace[2]);

		GLES20.glLineWidth(1);

		// Draw the x-axis.
		GLES20.glDrawArrays(GLES20.GL_LINES, 0, axesXPoints);
	}

	/**
	 * Draw the y-axis ticks.
	 * 
	 * @param mvpMatrix
	 *            the model-view-perspective matrix
	 * @param viewMatrix
	 *            the view matrix
	 * @param projectionMatrix
	 *            the projection matrix
	 * @param modelMatrix
	 *            the model matrix
	 * @param lightPosInEyeSpace
	 *            the light position in eye space matrix
	 */
	private void drawYAxis(float[] mvpMatrix, float[] viewMatrix,
			float[] projectionMatrix, float[] modelMatrix,
			float[] lightPosInEyeSpace)
	{
		// Draw the y-axis.
		GLES20.glUseProgram(lineProgramHandle);

		// Pass in the position information
		axesYVertexBuffer.position(0);
		GLES20.glVertexAttribPointer(axesYPositionHandle, positionDataSize,
				GLES20.GL_FLOAT, false, 0, axesYVertexBuffer);

		GLES20.glEnableVertexAttribArray(axesYPositionHandle);

		// Pass in the color information
		axesYColorBuffer.position(0);
		GLES20.glVertexAttribPointer(axesYColorHandle, colorDataSize,
				GLES20.GL_FLOAT, false, 0, axesYColorBuffer);

		GLES20.glEnableVertexAttribArray(axesYColorHandle);

		// Pass in the normal information
		axesYNormalsBuffer.position(0);
		GLES20.glVertexAttribPointer(axesYNormalHandle, normalsDataSize,
				GLES20.GL_FLOAT, false, 0, axesYNormalsBuffer);

		GLES20.glEnableVertexAttribArray(axesYNormalHandle);

		// This multiplies the view matrix by the model matrix, and stores the
		// result in the MVP matrix
		// (which currently contains model * view).
		Matrix.multiplyMM(mvpMatrix, 0, viewMatrix, 0, modelMatrix, 0);

		// Pass in the modelview matrix.
		GLES20.glUniformMatrix4fv(mvMatrixHandle, 1, false, mvpMatrix, 0);

		// This multiplies the modelview matrix by the projection matrix, and
		// stores the result in the MVP matrix
		// (which now contains model * view * projection).
		Matrix.multiplyMM(mvpMatrix, 0, projectionMatrix, 0, mvpMatrix, 0);

		// Pass in the combined matrix.
		GLES20.glUniformMatrix4fv(mvpMatrixHandle, 1, false, mvpMatrix, 0);

		// Pass in the light position in eye space.
		GLES20.glUniform3f(lightPosHandle, lightPosInEyeSpace[0],
				lightPosInEyeSpace[1], lightPosInEyeSpace[2]);

		GLES20.glLineWidth(1);

		// Draw the y-axis.
		GLES20.glDrawArrays(GLES20.GL_LINES, 0, axesYPoints);
	}

	/**
	 * Draw the z-axis ticks.
	 * 
	 * @param mvpMatrix
	 *            the model-view-perspective matrix
	 * @param viewMatrix
	 *            the view matrix
	 * @param projectionMatrix
	 *            the projection matrix
	 * @param modelMatrix
	 *            the model matrix
	 * @param lightPosInEyeSpace
	 *            the light position in eye space matrix
	 */
	private void drawZAxis(float[] mvpMatrix, float[] viewMatrix,
			float[] projectionMatrix, float[] modelMatrix,
			float[] lightPosInEyeSpace)
	{
		// Draw the z-axis.
		GLES20.glUseProgram(lineProgramHandle);

		// Pass in the position information
		axesZVertexBuffer.position(0);
		GLES20.glVertexAttribPointer(axesZPositionHandle, positionDataSize,
				GLES20.GL_FLOAT, false, 0, axesZVertexBuffer);

		GLES20.glEnableVertexAttribArray(axesZPositionHandle);

		// Pass in the color information
		axesZColorBuffer.position(0);
		GLES20.glVertexAttribPointer(axesZColorHandle, colorDataSize,
				GLES20.GL_FLOAT, false, 0, axesZColorBuffer);

		GLES20.glEnableVertexAttribArray(axesZColorHandle);

		// Pass in the normal information
		axesZNormalsBuffer.position(0);
		GLES20.glVertexAttribPointer(axesZNormalHandle, normalsDataSize,
				GLES20.GL_FLOAT, false, 0, axesZNormalsBuffer);

		GLES20.glEnableVertexAttribArray(axesZNormalHandle);

		// This multiplies the view matrix by the model matrix, and stores the
		// result in the MVP matrix
		// (which currently contains model * view).
		Matrix.multiplyMM(mvpMatrix, 0, viewMatrix, 0, modelMatrix, 0);

		// Pass in the modelview matrix.
		GLES20.glUniformMatrix4fv(mvMatrixHandle, 1, false, mvpMatrix, 0);

		// This multiplies the modelview matrix by the projection matrix, and
		// stores the result in the MVP matrix
		// (which now contains model * view * projection).
		Matrix.multiplyMM(mvpMatrix, 0, projectionMatrix, 0, mvpMatrix, 0);

		// Pass in the combined matrix.
		GLES20.glUniformMatrix4fv(mvpMatrixHandle, 1, false, mvpMatrix, 0);

		// Pass in the light position in eye space.
		GLES20.glUniform3f(lightPosHandle, lightPosInEyeSpace[0],
				lightPosInEyeSpace[1], lightPosInEyeSpace[2]);

		GLES20.glLineWidth(1);

		// Draw the z-axis.
		GLES20.glDrawArrays(GLES20.GL_LINES, 0, axesZPoints);
	}
}
